;***************************************************************************
;* AVR-Microcontroller als Videocontroller fr Grafik-LC-Displays          *
;* Simulation eines Oszilloskops mit langsam verblassendem Schreibstrahl   *
;* Das Video-Ram von 32 kByte ist in 4k-Blcke aufgeteilt: 4k Hardcopy-Ram *
;* 4k Skalen-Ram und 6*4k Video-Ram - nur dieses wird regelmig gelscht  *
;* Die Fading-Geschwindigkeit ist ber die Konstante FdConst whlbar       *
;* Christoph Kessler Mrz-April 2001                    (db1uq@t-online.de)*
;***************************************************************************
.nolist
.include "8515def.inc"
.list
;***************************************************************************
;* Register-Variablen 0 ( reserviert fr 16Bit-LPM-Befehl ):
;***************************************************************************
.def	LpmReg	= r00	; fr Tabellen-Operationen
;***************************************************************************
;* Register-Variablen 1-15 ( keine "immediate"-Befehle mglich ):
;***************************************************************************
.def	reg1	= r01	;
.def	SerDat	= r02	; Serielle Daten zur SPI-Ausgabe zwischenspeichern
.def	EffEff	= r03	; Register immer $FF, um Video-RAM zu lschen
.def	XWert	= r04	; Messwert X-Koordinate vom AD-Wandler
.def	YWert	= r05	; Messwert Y-Koordinate vom AD-Wandler
.def	PixByt	= r06	; Zwischenspeicher 1 Pixel innerhalb eines Bytes
.def	X_Circ	= r07	; X-Koordinate zur Circle-Routine
.def	Y_Circ	= r08	; Y-Koordinate zur Circle-Routine
.def	X_Pix	= r09	; X-Koordinate zur PutPixel-Routine
.def	Y_Pix	= r10	; Y-Koordinate zur PutPixel-Routine
.def	ImpFlg	= r11	; Flag "Vertikalimpuls fllig"
.def	reg12	= r12	; 
.def	reg13	= r13	; 
.def	reg14	= r14	; 
.def	reg15	= r15	; 
;***************************************************************************
;* Register-Variablen 16-25 ( "immediate"-Befehle mglich ):
;***************************************************************************
.def	X_Cent	= r16	; X-Koordinate Kreismittelpunkt (nur fr Init-Routine)
.def	Y_Cent	= r17	; Y-Koordinate Kreismittelpunkt (nur fr Init-Routine)
.def	Radius	= r18	; Kreis-Radius (nur fr Init-Routine)
.def	Segmt	= r19	; Auswahl der Achtel-Kreissegmente (nur fr Init-Routine)
.def	Cmp	= r16	; temporres Vergleichsregister (fr Interruptroutine)
.def	SvSREG	= r17	; Flagregister-Zwischenspeicher (fr Interruptroutine)
.def	FadCnt	= r18	; Zhler fr Video-Fading (fr Interruptroutine)
.def	DatL	= r19	; AD-Wandler-Daten low (ab Hauptprogramm)
.def	DatH	= r20	; AD-Wandler-Daten high (ab Hauptprogramm)
.def	Cnt	= r21	; Zhlvariable RAM-Bereiche (ab Hauptprogramm)
.def	XLo	= r22	; X-Koordinate LowByte (ab Hauptprogramm)
.def	XHi	= r23	; X-Koordinate HighByte (ab Hauptprogramm)
.def	reg24	= r24	;
.def	Temp	= r25	; temporres Register Hauptprogramm
;***************************************************************************
;* Register-Variablen 26-31 ( 16Bit-Register ) schon in "inc"-file definiert:
;***************************************************************************
;	XL	= r26	; Zeiger fr Interrupt-Medaten-RAM-Ausgabe low
;	XH	= r27	; Zeiger fr Interrupt-Medaten-RAM-Ausgabe high
;	YL	= r28	; Zeiger fr Interrupt-Skalen-RAM-Ausgabe low
;	YH	= r29	; Zeiger fr Interrupt-Skalen-RAM-Ausgabe high
;	ZL	= r30	; Zeiger auf SRAM low
;	ZH	= r31	; Zeiger auf SRAM high

;*************************************************************************
;* Konstanten-Definitionen:
;*************************************************************************
.equ	Vrt	= 4	; PortB Bit4 = Bildimpuls-Ausgang
.equ	Hor	= 3	; PortB Bit3 = Zeilenimpuls-Ausgang
.equ	SClk	= 4	; PortD Bit4 = Serial clock-Ausgang
.equ	Din	= 2	; PortB Bit2 = data input ADC = output AVR
.equ	Dout	= 5	; PortD Bit5 = data output ADC = input AVR
.equ	CS	= 1	; PortB Bit1 = /chip select
.equ	AD1	= $06	; Steuerwort AD-Kanal 1
.equ	AD2	= $16	; Steuerwort AD-Kanal 2
.equ	FdConst	= 7	; Konstante zum Video-Fading, Bedingung n*6

;*************************************************************************
;* Reset/Interrupt-Vektoren:
;*************************************************************************
.cseg
	rjmp	Initial		; nach RESET zum Hauptprogramm
	nop			; ExtInt0
	nop			; ExtInt1
	nop			; Timer1_capture
	nop			; Timer1_compareA
	nop			; Timer1_compareB
	nop			; Timer1_overflow
	nop			; Timer0_overflow

;**************************************************************************
;* Interrupt-Programm zum synchronen seriellen Port, beginnt direkt 
;* mit dem  SPI-INT-Vektor, das spart zeitlich einen rjmp-Befehl. 
;* Vier nachfolgende INT-Vektoren - UART und Komparator - entfallen damit
;* T-Flag =1  bedeutet, das Byte im Video-SRAM wird nach Ausgabe gelscht
;* bei konstanter Video-Datenrate von 2 MBit/sec tritt ein Interrupt alle
;* 32 Takte auf. Da die SPI-Schnittstelle der Atmel-AVRs keinen Sendepuffer
;* hat, verzgert sich die Ausgabe nach je 8 Bit abhngig von der Interrupt-
;* Routine um min. 12 bis max. 36 Takte. Die Taktanzahl jedes Befehls ist
;* daher hier in Klammern vermerkt
;**************************************************************************

SPIInt:				; (4) Takte fr Interrupt-Start
	in	SvSREG,SREG	; (1) Flagregister retten
	brts	Spcial		; (1+1) T-Flag=1 Spezialfall Hor/Vert-Impuls od.SkalenRam
	out	SPDR,SerDat	; (1) serielle Byte-Ausgabe starten
	mov	Cmp,XL		; (1) Hor-Zhlerstand holen
	andi	Cmp,$1F		; (1) 32 Byte erreicht, LowByte= % xxx0 0000 ?
	breq	LinEnd		; (1+1) Zeilenende erreicht
	dec	FadCnt		; (1) Fading counter
	breq	Fade		; (1+1)
	ld	SerDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - 12 von 32 Takten
LinEnd:
	sbr	SvSREG,$40	; (1) T=1, nchstes Mal Synchronimpuls ausgeben 
	mov	Cmp,XH		; (1) Vert-Zhlerstand holen
	andi	Cmp,$0F		; (1) 4 kByte erreicht, HighByte= % xxxx 0000?
	breq	Line4k		; (1+1) dann erst noch Lowbyte prfen
	clr	ImpFlg		; (1) ImpFlg=$00 -> nur Hor-Impuls fllig
	dec	FadCnt		; (1) Fading counter dekrementieren
	breq	Fade		; (1+1)
	ld	SerDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - 17 von 32 Takten

Line4k:
	cpi	XL,$00		; (1) 4 kByte erreicht, LowByte= % 0000 0000?
	breq	PicEnd		; (1+1)
	clr	ImpFlg		; (1) ImpFlg=$00 -> nchstes Mal Hor-Impuls
	dec	FadCnt		; (1) Fading counter dekrementieren
	breq	Fade		; (1+1)
	ld	SerDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - 20 von 32 Takten

PicEnd:
	clr	ImpFlg		; (1) ImpFlg=$00
	dec	ImpFlg		; (1) ImpFlg=$FF, ab nchstem Mal SkalenRAM 
	dec	ImpFlg		; (1) ImpFlg=$FE
	dec	ImpFlg		; (1) ImpFlg=$FD, nchstes Mal Vert.-Impuls 
	cpi	XH,$80		; (1) 32 kByte erreicht HighByte= % 1000 0000?
	breq	MemEnd		; (1+1) dann X-Zhler auf Medaten-Ram-Anfang
	ld	SerDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten

MemEnd:
	ldi	XH,$20		; (1) Zeiger auf Medaten-RAM-Anfang
	ld	SerDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten

Fade:
	ldi	FadCnt,FdConst	; (1) Bedingung n*6+1, hier n=1
	ld	SerDat,X	; (2) Medaten-Ausgabe-Byte holen
	st	X+,EffEff	; (2) Medaten-Byte im Ram lschen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx/xx/xx/xx von 32 Takten

;**************************************************************************
; Interruptausgabe Spezialflle Hor-Impuls, Vert-Impuls, Skalen-Ram
;**************************************************************************
Spcial:
	tst	ImpFlg		; (1) welcher Spezialfall ?
	brmi	ScaRam		; (1+1) Flag=$FF/$FE -> Skalen-RAM
	brne	VrtImp		; (1+1) Flag =01 -> Vert.-Impuls
	sbi	PortB,Hor	; (2) Zeilenimpuls _/~
	cbi	PortB,Hor	; (2) Zeilenimpuls ~\_
	out	SPDR,SerDat	; (1) serielle Byte-Ausgabe starten
	cbr	SvSREG,$40	; (1) T=0,kein Impuls nchstes Mal
	dec	FadCnt		; (1) Fading counter
	breq	Fade		; (1+1)
	ld	SerDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx Takte
VrtImp:
	sbi	PortB,Hor	; (2) Zeilenimpuls _/~
	sbi	PortB,Vrt	; (2) Bildimpuls _/~
	cbi	PortB,Hor	; (2) Zeilenimpuls ~\_
	cbi	PortB,Vrt	; (2) Bildimpuls ~\_
	out	SPDR,SerDat	; (1) serielle Byte-Ausgabe starten
	cbr	SvSREG,$40	; (1) T=0,kein Impuls nchstes Mal
	clr	ImpFlg		; (1) ImpFlg=$00 nchstes Mal Hor-Impuls
	dec	FadCnt		; (1) Fading counter
	breq	Fade		; (1+1)
	ld	SerDat,X+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx Takte
;**************************************************************************
; Interruptausgabe des Skalen-Ram im Wechsel mit den Medaten
;**************************************************************************
ScaRam:
	inc	ImpFlg		; (1) ImpFlg= $FE,$FF oder $00
	brmi	ScaImp		; (1+1) $FE/$FF -> Synchronimpuls fllig
	out	SPDR,SerDat	; (1) serielle Byte-Ausgabe starten
	dec	ImpFlg		; (1) ImpFlg zurck auf $FF
	mov	Cmp,YL		; (1) Hor-Zhlerstand holen
	andi	Cmp,$1F		; (1) 32 Byte erreicht, LowByte= % xxx0 0000 ?
	breq	Scal4k		; (1+1) Zeilenende erreicht
	ld	SerDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten
Scal4k:
	cpi	YH,$20		; (1) 4 kByte erreicht, HighByte= % 0020 0000?
	breq	ScaEnd		; (1+1) dann nchstes Mal Vertikalimpuls fllig
	dec	ImpFlg		; (1) ImpFlg=$FE -> nchstes Mal Hor-Impuls fllig
	ld	SerDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten
ScaEnd:
	ldi	YH,$10		; (1) Zeiger auf Skalen-RAM-Anfang
	clr	ImpFlg		; (1) ab nchstem Mal wieder Medatenausgabe
	inc	ImpFlg		; (1) ImpFlg=$01 -> nchstes Mal Vertikalimpuls
	sbr	SvSREG,$40	; (1) T=1, fr Vert.-Impuls
	dec	FadCnt		; (1) Fading counter
	breq	Fade		; (1+1)
	ld	SerDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten

ScaImp:
	inc	ImpFlg		; (1) jetzt: $FF =Vert, $00 =Hor
	brmi	ScaVrt		; (1+1) Flag =$FF -> Vert.-Impuls
	sbi	PortB,Hor	; (2) Zeilenimpuls _/~
	cbi	PortB,Hor	; (2) Zeilenimpuls ~\_
	out	SPDR,SerDat	; (1) serielle Byte-Ausgabe starten
	dec	ImpFlg		; (1) ImpFlg=$FF kein Impuls nchstes Mal
	ld	SerDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx Takte
ScaVrt:
	sbi	PortB,Hor	; (2) Zeilenimpuls _/~
	sbi	PortB,Vrt	; (2) Bildimpuls _/~
	cbi	PortB,Hor	; (2) Zeilenimpuls ~\_
	cbi	PortB,Vrt	; (2) Bildimpuls ~\_
	out	SPDR,SerDat	; (1) serielle Byte-Ausgabe starten
	ld	SerDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx Takte

;*************************************************************************
;* nach Reset: Stack,Ports,ext.SRAM,Interrupts initialisieren
;*************************************************************************
Initial: 
	ldi	Temp,Low(RamEnd)
	out	spl,Temp	;
	ldi	Temp,High(RamEnd)
	out	sph,Temp	; Stack initialisiert

	clr	Temp		;
	out	PortA,Temp	;
	out	PortB,Temp	;
	out	PortC,Temp	;
	out	PortD,Temp	;
	out	DDRA,Temp	; (egal,vom SRAM belegt)
	out	DDRC,Temp	; (egal,vom SRAM belegt)
	ldi	Temp,$BE	; T0Zhlinp.,CS_ADC,/Hor/Vrt out,SPI-pins
	out	DDRB,Temp	; 
	ldi	Temp,$DF	; RS232in/out,INT0/1-in, PD4=VrtSync output,
	out	DDRD,Temp	; PD5=input vom ADC, PD6/7 vom SRAM belegt

	ldi	Temp,$8A	; ext.SRAM enable, 1 waitstate,Int0/1 _/~
	out	MCUCR,Temp	; MicroControllerUnit Control Reg.

	clr	EffEff		;
	dec	EffEff		; weier Hintergrund =$FF

	clr	XL
	clr	XH		; Zeiger auf SRAM-Anfang

FillMem:
	st	X+,EffEff	; gesamtes SRam lschen, 
	cpi	XH,$80		;
	brne	FillMem

;*************************************************************************
;* Skalen zeichnen
;*************************************************************************
;* untere Display-Hlfte hor.Skalenstriche:
	clr	XL
	ldi	XH,$10		; Zeiger auf Skalen-Ram (SRAM-Anfang+4kByte)
	ldi	Temp,$FE
Scale:
	ldi	Cnt,4
Scale1:
	st	X+,Temp		; abwechselnd $FE $EF $FF = 12Bit/div
	swap	Temp
	st	X+,Temp		;
	swap	Temp
	st	X+,EffEff	;
	dec	Cnt		; nach 12 Byte nchste Zeile
	brne	Scale1
	andi	XL,$E0		; 32 Byte/Zeile, erst zurck zum Zeilenanfang
	adiw	XL,$20		; und dann 16Bit inkrementieren
	cpi	XH,$20
	brne	Scale

;* Jetzt Smith-Diagramm in der oberen Display-Hlfte:

	ldi	Segmt,$FF	; Vollkreis zeichnen
	ldi	Radius,63	; Radius = 63
	ldi	X_Cent,176	; X-Center = 176
	ldi	Y_Cent,63	; Y-Center = 63
	rcall	Circle		;
	ldi	Segmt,$30	; Segmente 4 und 5 zeichnen
	ldi	X_Cent,113	; X-Center = 113
	ldi	Y_Cent,126	; Y-Center = 126
	rcall	Circle		;
	ldi	Segmt,$C0	; Segmente 6 und 7 zeichnen
	ldi	X_Cent,239	; X-Center = 239
	ldi	Y_Cent,126	; Y-Center = 126
	rcall	Circle		;
	ldi	Segmt,$FF	; Vollkreis zeichnen
	ldi	Radius,47	; Radius = 47
	ldi	X_Cent,176	; X-Center = 176
	ldi	Y_Cent,79	; Y-Center = 79
	rcall	Circle		;
	ldi	Segmt,$FF	; Vollkreis zeichnen
	ldi	Radius,31	; Radius = 31
	ldi	X_Cent,176	; X-Center = 176
	ldi	Y_Cent,95	; Y-Center = 95
	rcall	Circle		;
	ldi	Segmt,$FF	; Vollkreis zeichnen
	ldi	Radius,15	; Radius = 15
	ldi	X_Cent,176	; X-Center = 176
	ldi	Y_Cent,111	; Y-Center = 111
	rcall	Circle		;
	ldi	Segmt,$10	; Segment 4 zeichnen
	ldi	Radius,127	; Radius = 127
	ldi	X_Cent,48	; X-Center = 48
	ldi	Y_Cent,120	; Y-Center = 120
	rcall	Circle		;
	ldi	Segmt,$80	; Segment 7 zeichnen
	ldi	Radius,127	; Radius = 127
	ldi	X_Cent,48	; X-Center = 48
	ldi	Y_Cent,120	; Y-Center = 120
	rcall	Circle		;

;*************************************************************************
;* Interrupt starten
;*************************************************************************
	clr	XL
	clr	YL
	ldi	XH,$20
	ldi	YH,$10

	ldi	Temp,$F4	; LSB zuerst,Clk/4,aktiv=~\_, Ruhe=0
	out	SPCR,Temp	; Serial Periph.Interface Control Reg.

	ldi	FadCnt,FdConst	; Video-Fading starten
	clt			; T-Flag=0 -> Skalen-Ram nicht lschen
	sei			; Interrupt Enable Flag

	out	SPDR,EffEff	; Dummy zum SPI-Start ausgeben


;*************************************************************************
;* Hauptprogramm 
;*************************************************************************
Haupt:
	cbi	PortB,CS	; AD-Wandler ChipSelect ~\_ , bis Dout
				; min 2 AClk-Cycles = 1,08 Sec warten
	ldi	DatH,AD2	; (1) AD-Kanal 2 anfordern, und
	rcall	ADCin		; (3) vorherigen AD-Wert1 lesen

	ldi	Temp,100	; 52 AClk-cycles warten = 113 4MHz-Takte
Warte:
	dec	Temp		; (1)
	brne	Warte		; (2/1)
	ldi	Temp,$80	; Zweierkomplement in unsigned Wert umwandeln
	eor	DatH,Temp
	mov	Temp,DatH	; (1) X-Koordinate zwischenspeichern
	lsr	DatH		; (1) Aufteilen auf 8 Pixel * 32 Byte pro Zeile
	lsr	DatH		; (1)
	lsr	DatH		; (1)
	mov	XWert,DatH	; (1) 5 Bit Byteadresse in XWert
	andi	Temp,$07	; (1) untere 3 Bit ergeben Pixelposition
	clr	PixByt		; (1) auf Null
	sec			; (1) Pixel zunchst ins Cy
Pixloop:
	rol	PixByt		; (1) Pixel weiterschieben
	dec	Temp		; (1) 
	brge	Pixloop		; (1+1) 
	com	PixByt		; (1) schwarzes Pixel auf weiem Hintergrund

	ldi	Temp,100		; 52 AClk-cycles warten = 113 4MHz-Takte
Wart1:
	dec	Temp		; (1)
	brne	Wart1		; (2/1)

	cbi	PortB,CS	; AD-Wandler ChipSelect ~\_ , bis Dout
				; min 2 AClk-Cycles = 1,08 Sec warten
	ldi	DatH,AD1	; (1) AD-Kanal 1 anfordern, und
	rcall	ADCin		; (3) vorherigen AD-Wert2 lesen
	ldi	Temp,$80	; Zweierkomplement in unsigned Wert umwandeln
	eor	DatH,Temp

	mov	YWert,DatH	; Y-Originalwert aufbewahren

	clr	ZL		; SRam-Adresse berechnen
	lsr	DatH		; LSBit verwerfen
	lsr	DatH		; 7Bit-Y-Koordinate aufteilen in 4+3 Bit
	ror	ZL		; nchste 3 Bit von links in ZL schieben
	lsr	DatH		;
	ror	ZL		;
	lsr	DatH		;
	ror	ZL		;
	add	ZL,XWert	; 5 Bit X-Wert addieren
	ldi	ZH,$20		; erstes Video-Ram $2000..$2FFF
	add	ZH,DatH		; obere 4 Bit der Y-Koordinate addieren

	ldi	Cnt,6
VidRam:
	ld	Temp,Y		; 8 Pixel holen
	and	Temp,PixByt	; andere Bits unverndert lassen
	st	Z,Temp
	subi	ZH,-$10		; (=Add Immediate $10) $3000..3FFF usw.
	dec	Cnt
	brne	VidRam

	rjmp	Haupt		; Hauptprogramm = ewige Warteschleife	
;*************************************************************************

;*************************************************************************
;* Unterprogramm liest vom AD-Wandler LTC1290 12 Bit seriell aus
;* gleichzeitig serielle Anforderung des nchsten AD-Wertes
;* beide Werte stehen im gleichen Registerpaar 
;*************************************************************************
ADCin:
	ldi	Temp,10		; 
Warte2:
	dec	Temp		; (1)
	brne	Warte2		; (2/1)
	clr	DatL		; (1) DatL=0
	sec			; (1) Cy=1
	rol	DatL		; (1) Cy=0, DatL=01h - Zhler fr 8 Datenbits
Loop8:
	sbic	PinD,Dout	; nchste Zeile berspringen wenn Dout= 0
	sec			; Dout=1 also Cy=1
	rol	DatH		; Dout ins unterste Bit schieben
	brcc	low		; oberstes Bit via Cy zum ADC-Din
	sbi	PortB,Din	; Din=H
	brcs	high
low:
	cbi	PortB,Din	; Din=L
high:
	sbi	PortD,SClk	; ADC-Clock _/~
	cbi	PortD,SClk	; ADC-Clock ~\_
	lsl	DatL		; Zhler weiterschieben
	brcc	Loop8		; 8 Bit erreicht ?

	rol	DatL		; (vorher Cy=1, DatL=00h) Cy=0, DatL=01h
	swap	DatL		; DatL=10h, Doppelfunktion: Zhler und Daten
Loop4:
	sbic	PinD,Dout	; nchste Zeile berspringen wenn Dout= 0
	sec			; Dout=1 also Cy=1
	rol	DatL		; Dout ins unterste Bit schieben
	sbi	PortD,SClk	; ADC-Clock _/~
	cbi	PortD,SClk	; ADC-Clock ~\_
	brcc	Loop4		; 4 Bit erreicht ?
	swap	DatL		; DatL=0000xxxx --> xxxx0000
	sbi	PortB,CS	; AD-Wandler ChipSelect _/~
	ret

;*************************************************************************
;* Unterprogramm zur Ausgabe eines oder mehrerer Achtel-Kreis-Segmente 
;* (Bresenhams Algorithmus) bergabevariable X_Cent,Y_Cent,Radius,Segmt
;*************************************************************************
Circle:
	clr	X_Circ		; Startwert X=0
	mov	Y_Circ,Radius	; Startwert Y=Radius
	clr	XH
	ldi	XL,3		; D=3
	clr	ZH		; 
	mov	YL,Radius	; 
	lsl	YL		; Radius * 2
	rol	YH		;
	sub	XL,YL		; D=D-2*Radius
	sbc	XH,YH		;
CircLoop:
	rcall	Segm0		;
	inc	X_Circ		; 
	tst	XH		; D>0  ?
	brpl	Ydec		; dann Y decrementieren
	adiw	XL,6		; D=D+6
	clr	YH		;
	mov	YL,X_Circ	; 
	lsl	YL		; X*4
	rol	YH		;
	lsl	YL		; 
	rol	YH		;
	add	XL,YL		; D=D+4*x
	adc	XH,YH		; 
	cp	Y_Circ,X_Circ	;
	brge	CircLoop	; usw. bis X=Y
	ret
	
Ydec:
	adiw	XL,10		; D=D+10
	clr	YH		; x immer kleiner als y
	mov	YL,X_Circ	; aber beide positiv
	sub	YL,Y_Circ	; 16Bit-Subtraktion x-y
	sbci	YH,0		; immer negatives Ergebnis bis Schleifenende
	lsl	YL		; (x-y)*4
	rol	YH		;
	lsl	YL		; 
	rol	YH		;
	add	XL,YL		; D=D+4*(x-y)
	adc	XH,YH		;
	dec	Y_Circ		; y=y-1
	cp	Y_Circ,X_Circ
	brge	CircLoop	; usw. bis X=Y
	ret
;*************************************************************************
;* Unterprogramm zur Ausgabe von einem bis acht Achtelkreissegmentpixeln
;* bergabevariable X_Circ,Y_Circ,X_Cent,Y_Cent,Segmt  
;* Zhlweise Segmente - Koordinaten(X/Y):
;* (239/127)  I~~~~~~~~~~~~~~~I (0/127)
;*            I   2 1         I
;*            I  3   0        I
;*            I  4   7        I
;*            I   5 6         I
;* (239/0)    I_______________I (0/0)
;*************************************************************************
Segm0:	
	sbrs	Segmt,0		; Achtel 0 zeichnen ?
	rjmp	Segm1
	mov	X_Pix,X_Cent	; Kreismittelpunkt X_Cent
	sub	X_Pix,Y_Circ	; minus Y-Koordinate Y_Circ
	mov	Y_Pix,Y_Cent	; Kreismittelpunkt Y_Cent
	add	Y_Pix,X_Circ	; plus X-Koordinate X_Circ
	rcall	PutPix		; Bildpunkt auf Display ausgeben
Segm1:	
	sbrs	Segmt,1		; Achtel 1 zeichnen ?
	rjmp	Segm2
	mov	X_Pix,X_Cent	; Kreismittelpunkt X_Cent
	sub	X_Pix,X_Circ	; minus X-Koordinate X_Circ
	mov	Y_Pix,Y_Cent	; Kreismittelpunkt Y_Cent
	add	Y_Pix,Y_Circ	; plus Y-Koordinate Y_Circ
	rcall	PutPix		; Bildpunkt auf Display ausgeben
Segm2:
	sbrs	Segmt,2		; Achtel 2 zeichnen ?
	rjmp	Segm3
	mov	X_Pix,X_Cent	; Kreismittelpunkt X_Cent
	add	X_Pix,X_Circ	; plus X-Koordinate X_Circ
	mov	Y_Pix,Y_Cent	; Kreismittelpunkt Y_Cent
	add	Y_Pix,Y_Circ	; plus Y-Koordinate Y_Circ
	rcall	PutPix		; Bildpunkt auf Display ausgeben
Segm3:	
	sbrs	Segmt,3		; Achtel 3 zeichnen ?
	rjmp	Segm4
	mov	X_Pix,X_Cent	; Kreismittelpunkt X_Cent
	add	X_Pix,Y_Circ	; plus Y-Koordinate Y_Circ
	mov	Y_Pix,Y_Cent	; Kreismittelpunkt Y_Cent
	add	Y_Pix,X_Circ	; plus X-Koordinate X_Circ
	rcall	PutPix		; Bildpunkt auf Display ausgeben
Segm4:	
	sbrs	Segmt,4		; Achtel 4 zeichnen ?
	rjmp	Segm5
	mov	X_Pix,X_Cent	; Kreismittelpunkt X_Cent
	add	X_Pix,Y_Circ	; plus Y-Koordinate Y_Circ
	mov	Y_Pix,Y_Cent	; Kreismittelpunkt Y_Cent
	sub	Y_Pix,X_Circ	; minus X-Koordinate X_Circ
	rcall	PutPix		; Bildpunkt auf Display ausgeben
Segm5:	
	sbrs	Segmt,5		; Achtel 5 zeichnen ?
	rjmp	Segm6
	mov	X_Pix,X_Cent	; Kreismittelpunkt X_Cent
	add	X_Pix,X_Circ	; plus X-Koordinate X_Circ
	mov	Y_Pix,Y_Cent	; Kreismittelpunkt Y_Cent
	sub	Y_Pix,Y_Circ	; minus Y-Koordinate Y_Circ
	rcall	PutPix		; Bildpunkt auf Display ausgeben
Segm6:	
	sbrs	Segmt,6		; Achtel 6 zeichnen ?
	rjmp	Segm7
	mov	X_Pix,X_Cent	; Kreismittelpunkt X_Cent
	sub	X_Pix,X_Circ	; minus X-Koordinate X_Circ
	mov	Y_Pix,Y_Cent	; Kreismittelpunkt Y_Cent
	sub	Y_Pix,Y_Circ	; minus Y-Koordinate Y_Circ
	rcall	PutPix		; Bildpunkt auf Display ausgeben
Segm7:	
	sbrs	Segmt,7		; Achtel 7 zeichnen ?
	ret
	mov	X_Pix,X_Cent	; Kreismittelpunkt X_Cent
	sub	X_Pix,Y_Circ	; minus Y-Koordinate Y_Circ
	mov	Y_Pix,Y_Cent	; Kreismittelpunkt Y_Cent
	sub	Y_Pix,X_Circ	; minus X-Koordinate X_Circ
	rcall	PutPix		; Bildpunkt auf Display ausgeben
	ret

;*************************************************************************
;* Unterprogramm schreibt ein Pixel ins Skalen-Ram, restliche Pixel
;* bleiben unberhrt, bergabevariable X_Pix, Y_Pix je ein Byte (MSB von Y=0)
;*************************************************************************
PutPix:
	mov	Temp,X_Pix	; (1) Aufteilen: 8 Pixel * 32 Byte pro Zeile
	lsr	X_Pix		; (1) 
	lsr	X_Pix		; (1)
	lsr	X_Pix		; (1) 5 Bit Byteadresse in X_Pix
	andi	Temp,$07	; (1) untere 3 Bit ergeben Pixelposition
	clr	PixByt		; (1) auf Null
	sec			; (1) Pixel zunchst ins Cy
PixRol:
	rol	PixByt		; (1) Pixel weiterschieben
	dec	Temp		; (1) 
	brge	PixRol		; (1+1) 
	com	PixByt		; (1) schwarzes Pixel auf weiem Hintergrund
	clr	YL		; SRam-Adresse berechnen
	lsr	Y_Pix		; 7Bit-Y-Koordinate aufteilen in 4+3 Bit
	ror	YL		; nchste 3 Bit von links in YL schieben
	lsr	Y_Pix		;
	ror	YL		;
	lsr	Y_Pix		;
	ror	YL		;
	add	YL,X_Pix	; 5 Bit X-Wert addieren
	ldi	YH,$10		; Skalen-Ram $1000..$1FFF
	add	YH,Y_Pix	; obere 4 Bit der Y-Koordinate addieren

	ld	Temp,Y		; 8 Pixel holen
	and	Temp,PixByt	; andere Bits unverndert lassen
	st	Y,Temp
	ret

;*************************************************************************
;* Textpuffer im SRAM
;*************************************************************************
.dseg
.org $60
TxtBuf:
.BYTE 16
ChrBuf:
.BYTE 64
