;***************************************************************************
;* AVR-Microcontroller als Videocontroller fr Grafik-LC-Displays          *
;* Simulation eines Oszilloskops mit langsam verblassendem Schreibstrahl   *
;* Das Video-Ram von 32 kByte ist in 4k-Blcke aufgeteilt: 4k Hardcopy-Ram *
;* 4k Skalen-Ram und 6*4k Video-Ram - nur dieses wird regelmig gelscht  *
;* Die Fading-Geschwindigkeit ist ber die Konstante FdConst whlbar       *
;* Ein D/A-Wandler liefert eine Dreieckspannung, ein A/D-Wandler mit die  *
;* Detektorspannung, Displayformat 480 x 64 Pixel                          *
;* Christoph Kessler August 2001                        (db1uq@t-online.de)*
;***************************************************************************
.nolist
.include "8515def.inc"
.list
;***************************************************************************
;* Register-Variablen 0 ( reserviert fr 16Bit-LPM-Befehl ):
;***************************************************************************
.def	LpmReg	= r00	; fr Tabellen-Operationen
;***************************************************************************
;* Register-Variablen 1-15 ( keine "immediate"-Befehle mglich ):
;***************************************************************************
.def	EffEff	= r01	; Register immer $FF
.def	LCDDat	= r02	; Serielle Daten zur SPI-Ausgabe zwischenspeichern
.def	NulNul	= r03	; Register immer $00 , um Video-RAM zu lschen
.def	XWert	= r04	; Messwert X-Koordinate vom AD-Wandler
.def	YWert	= r05	; Messwert Y-Koordinate vom AD-Wandler
.def	PixByt	= r06	; Zwischenspeicher 1 Pixel innerhalb eines Bytes
.def	X_Circ	= r07	; X-Koordinate zur Circle-Routine
.def	Y_Circ	= r08	; Y-Koordinate zur Circle-Routine
.def	X_Pix	= r09	; X-Koordinate zur PutPixel-Routine
.def	Y_Pix	= r10	; Y-Koordinate zur PutPixel-Routine
.def	ImpFlg	= r11	; Flag "Vertikalimpuls fllig"
.def	reg12	= r12	; 
.def	reg13	= r13	; 
.def	reg14	= r14	; 
.def	reg15	= r15	; 
;***************************************************************************
;* Register-Variablen 16-25 ( "immediate"-Befehle mglich ):
;***************************************************************************
.def	Cmp	= r16	; temporres Vergleichsregister (fr Interruptroutine)
.def	SvSREG	= r17	; Flagregister-Zwischenspeicher (fr Interruptroutine)
.def	FadCnt	= r18	; Zhler fr Video-Fading (fr Interruptroutine)
.def	ADDat	= r19	; AD-Wandler-Daten (ab Hauptprogramm)
.def	Char	= r20	; Zeichen zur Ausgabe ins Skalen-Ram (nur Init-Programm)
.def	Cnt	= r21	; Zhlvariable RAM-Bereiche (ab Hauptprogramm)
.def	XLo	= r22	; X-Koordinate LowByte (ab Hauptprogramm)
.def	XHi	= r23	; X-Koordinate HighByte (ab Hauptprogramm)
.def	Temp2	= r24	;
.def	Temp	= r25	; temporres Register Hauptprogramm
;***************************************************************************
;* Register-Variablen 26-31 ( 16Bit-Register ) schon in "inc"-file definiert:
;***************************************************************************
;	XL	= r26	; Zeiger fr Interrupt-Daten-SRAM-Ausgabe low
;	XH	= r27	; Zeiger fr Interrupt-Daten-SRAM-Ausgabe high
;	YL	= r28	; Zeiger fr Interrupt-Skalen-SRAM-Ausgabe low
;	YH	= r29	; Zeiger fr Interrupt-Skalen-SRAM-Ausgabe high
;	ZL	= r30	; Zeiger auf SRAM low
;	ZH	= r31	; Zeiger auf SRAM high

;*************************************************************************
;* Konstanten-Definitionen:
;*************************************************************************
.equ	Vrt	= 4	; PortB Bit4 = Bildimpuls-Ausgang
.equ	Hor	= 3	; PortB Bit3 = Zeilenimpuls-Ausgang
.equ	SClk	= 4	; PortD Bit4 = Serial clock-Ausgang
.equ	SDat	= 2	; PortB Bit2 = data input ADC und output DAC
.equ	CsAdc	= 5	; PortD Bit5 = /chip select ADC
.equ	LdDac	= 1	; PortB Bit1 = /Load DAC
.equ	LCDClk	= 3	; PortD Bit3 = "M"-clock
.equ	AD1	= $06	; Steuerwort AD-Kanal 1
.equ	AD2	= $16	; Steuerwort AD-Kanal 2
.equ	FdConst	= 7	; Video-Fading, nicht durch 2 oder 3 teilbar !
.equ	LinLen	= 64	; Lnge einer Displayzeile in Byte
;*************************************************************************
;* Reset/Interrupt-Vektoren:
;*************************************************************************
.cseg
	rjmp	Initial		; nach RESET zum Hauptprogramm
	nop			; ExtInt0
	nop			; ExtInt1
	nop			; Timer1_capture
	nop			; Timer1_compareA
	nop			; Timer1_compareB
	nop			; Timer1_overflow
	nop			; Timer0_overflow

;**************************************************************************
;* Interrupt-Programm zum synchronen seriellen Port, beginnt direkt 
;* mit dem  SPI-INT-Vektor, das spart zeitlich einen rjmp-Befehl. 
;* Vier nachfolgende INT-Vektoren - UART und Komparator - entfallen damit
;* bei konstanter Video-Datenrate von 2 MBit/sec tritt ein Interrupt alle
;* 32 Takte auf. Da die SPI-Schnittstelle der Atmel-AVRs keinen Sendepuffer
;* hat, verzgert sich die Ausgabe nach je 8 Bit abhngig von der Interrupt-
;* Routine um min. 12 bis max. 36 Takte. Die Taktanzahl jedes Befehls ist
;* daher hier in Klammern vermerkt
;**************************************************************************

SPIInt:				; (4) Takte fr Interrupt-Start
	in	SvSREG,SREG	; (1) Flagregister retten
	brts	Spcial		; (1+1) T-Flag=1 Spezialfall Hor/Vert-Impuls od.SkalenRam
	out	SPDR,LCDDat	; (1) serielle Byte-Ausgabe starten
	mov	Cmp,XL		; (1) Hor-Zhlerstand holen
	andi	Cmp,$3F		; (1) 64 Byte erreicht, LowByte= % xx00 0000 ?
	breq	LinEnd		; (1+1) Zeilenende erreicht
	dec	FadCnt		; (1) Fading counter
	breq	Fade		; (1+1)
	ld	LCDDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - 12 von 32 Takten
LinEnd:
	sbr	SvSREG,$40	; (1) T=1, nchstes Mal Synchronimpuls ausgeben 
	mov	Cmp,XH		; (1) Vert-Zhlerstand holen
	andi	Cmp,$0F		; (1) 4 kByte erreicht, HighByte= % xxxx 0000?
	breq	Line4k		; (1+1) dann erst noch Lowbyte prfen
	clr	ImpFlg		; (1) ImpFlg=$00 -> nur Hor-Impuls fllig
	dec	FadCnt		; (1) Fading counter dekrementieren
	breq	Fade		; (1+1)
	ld	LCDDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - 17 von 32 Takten

Line4k:
	cpi	XL,$00		; (1) 4 kByte erreicht, LowByte= % 0000 0000?
	breq	PicEnd		; (1+1)
	clr	ImpFlg		; (1) ImpFlg=$00 -> nchstes Mal Hor-Impuls
	dec	FadCnt		; (1) Fading counter dekrementieren
	breq	Fade		; (1+1)
	ld	LCDDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - 20 von 32 Takten

PicEnd:
	clr	ImpFlg		; (1) ImpFlg=$00
	dec	ImpFlg		; (1) ImpFlg=$FF, ab nchstem Mal SkalenRAM 
	dec	ImpFlg		; (1) ImpFlg=$FE
	dec	ImpFlg		; (1) ImpFlg=$FD, nchstes Mal Vert.-Impuls 
	cpi	XH,$80		; (1) 32 kByte erreicht HighByte= % 1000 0000?
	breq	MemEnd		; (1+1) dann X-Zhler auf Medaten-Ram-Anfang
	ld	LCDDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten

MemEnd:
	ldi	XH,$20		; (1) Zeiger auf Medaten-RAM-Anfang
	ld	LCDDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten

Fade:
	ldi	FadCnt,FdConst	; (1) Bedingung n*6+1, hier n=1
	ld	LCDDat,X	; (2) Medaten-Ausgabe-Byte holen
	st	X+,NulNul	; (2) Medaten-Byte im Ram lschen
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx/xx/xx/xx von 32 Takten

;**************************************************************************
; Interruptausgabe Spezialflle Hor-Impuls, Vert-Impuls, Skalen-Ram
;**************************************************************************
Spcial:
	tst	ImpFlg		; (1) welcher Spezialfall ?
	brmi	ScaRam		; (1+1) Flag=$FF/$FE -> Skalen-RAM
	brne	VrtImp		; (1+1) Flag =01 -> Vert.-Impuls
	sbi	PortB,Hor	; (2) Zeilenimpuls _/~
	cbi	PortB,Hor	; (2) Zeilenimpuls ~\_
	out	SPDR,LCDDat	; (1) serielle Byte-Ausgabe starten
	cbr	SvSREG,$40	; (1) T=0,kein Impuls nchstes Mal
	dec	FadCnt		; (1) Fading counter
	breq	Fade		; (1+1)
	ld	LCDDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx Takte
VrtImp:
	sbi	PortB,Hor	; (2) Zeilenimpuls _/~
	sbi	PortB,Vrt	; (2) Bildimpuls _/~
	cbi	PortB,Hor	; (2) Zeilenimpuls ~\_
	cbi	PortD,LCDClk	; (2) LCD-Takt ~\_
	cbi	PortB,Vrt	; (2) Bildimpuls ~\_
	out	SPDR,LCDDat	; (1) serielle Byte-Ausgabe starten
	cbr	SvSREG,$40	; (1) T=0,kein Impuls nchstes Mal
	clr	ImpFlg		; (1) ImpFlg=$00 nchstes Mal Hor-Impuls
	dec	FadCnt		; (1) Fading counter
	breq	Fade		; (1+1)
	ld	LCDDat,X+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx Takte
;**************************************************************************
; Interruptausgabe des Skalen-Ram im Wechsel mit den Medaten
;**************************************************************************
ScaRam:
	inc	ImpFlg		; (1) ImpFlg= $FE,$FF oder $00
	brmi	ScaImp		; (1+1) $FE/$FF -> Synchronimpuls fllig
	out	SPDR,LCDDat	; (1) serielle Byte-Ausgabe starten
	dec	ImpFlg		; (1) ImpFlg zurck auf $FF
	mov	Cmp,YL		; (1) Hor-Zhlerstand holen
	andi	Cmp,$3F		; (1) 64 Byte erreicht, LowByte= % xx00 0000 ?
	breq	Scal4k		; (1+1) Zeilenende erreicht
	ld	LCDDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten
Scal4k:
	cpi	YH,$20		; (1) 4 kByte erreicht, HighByte= % 0020 0000?
	breq	ScaEnd		; (1+1) dann nchstes Mal Vertikalimpuls fllig
	dec	ImpFlg		; (1) ImpFlg=$FE -> nchstes Mal Hor-Impuls fllig
	ld	LCDDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten
ScaEnd:
	ldi	YH,$10		; (1) Zeiger auf Skalen-RAM-Anfang
	clr	ImpFlg		; (1) ab nchstem Mal wieder Medatenausgabe
	inc	ImpFlg		; (1) ImpFlg=$01 -> nchstes Mal Vertikalimpuls
	sbr	SvSREG,$40	; (1) T=1, fr Vert.-Impuls
	dec	FadCnt		; (1) Fading counter
	breq	Fade		; (1+1)
	ld	LCDDat,X+	; (2) Medaten-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx von 32 Takten

ScaImp:
	inc	ImpFlg		; (1) jetzt: $FF =Vert, $00 =Hor
	brmi	ScaVrt		; (1+1) Flag =$FF -> Vert.-Impuls
	sbi	PortB,Hor	; (2) Zeilenimpuls _/~
	cbi	PortB,Hor	; (2) Zeilenimpuls ~\_
	out	SPDR,LCDDat	; (1) serielle Byte-Ausgabe starten
	dec	ImpFlg		; (1) ImpFlg=$FF kein Impuls nchstes Mal
	ld	LCDDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx Takte
ScaVrt:
	sbi	PortB,Hor	; (2) Zeilenimpuls _/~
	sbi	PortB,Vrt	; (2) Bildimpuls _/~
	cbi	PortB,Hor	; (2) Zeilenimpuls ~\_
	sbi	PortD,LCDClk	; (2) LCD-Takt _/~
	cbi	PortB,Vrt	; (2) Bildimpuls ~\_
	out	SPDR,LCDDat	; (1) serielle Byte-Ausgabe starten
	ld	LCDDat,Y+	; (2) Skalen-Ausgabe-Byte holen, inkrem.
	out	SREG,SvSREG	; (1) Flagregister holen
	reti			; (4) fertig - xx Takte

;*************************************************************************
;* nach Reset: Stack,Ports,ext.SRAM,Interrupts initialisieren
;*************************************************************************
Initial: 
	ldi	Temp,Low(RamEnd)
	out	spl,Temp	;
	ldi	Temp,High(RamEnd)
	out	sph,Temp	; Stack initialisiert

	clr	Temp		;
	out	PortA,Temp	;
	out	PortB,Temp	;
	out	PortC,Temp	;
	out	PortD,Temp	;
	out	DDRA,Temp	; (egal,vom SRAM belegt)
	out	DDRC,Temp	; (egal,vom SRAM belegt)
	ldi	Temp,$BE	; PortB Direction = % 1011 1111
	out	DDRB,Temp	; T0Zhlinp.,CS_ADC,/Hor/Vrt out,SPI-pins
	ldi	Temp,$FF	; PortD Direction = % 1111 1111
	out	DDRD,Temp	; RS232in/out,INT0/1-in, PD4=VrtSync output,
				; PD5=/CS zumADC, PD6/7 vom SRAM belegt
	ldi	Temp,$8A	; ext.SRAM enable, 1 waitstate,Int0/1 _/~
	out	MCUCR,Temp	; MicroControllerUnit Control Reg.

	sbi	PortD,CsAdc	; ADC-/Enable _/~
	sbi	DDRB,SDat	; SDat = output

	ldi	XH,$20		; Anfangswerte
	clr	XL		; 
	ldi	YH,$10		; 
	clr	YL		; 
	clr	EffEff		; 
	dec	EffEff		; 
	clr	NulNul		; 

	clr	ZL
	ldi	ZH,$10		; Zeiger auf Medaten-Ram-Anfang
FillMem:
	st	Z+,NulNul	; gesamtes Medaten-Ram mit $00 fllen 
	cpi	ZH,$80		;
	brne	FillMem

;*************************************************************************
;* Skalenstriche zeichnen 
;*************************************************************************
	clr	YL		; Skalen-Ram
	ldi	YH,$13		; 
	ldi	Temp,$08	;
Scale:
	ldi	Cnt,12		;
Scale1:
	st	Y+,Temp		; je 5 Byte: $00 $10 $00 $01 $00 -> 20 Bit/div
	swap	Temp		:
	st	Y+,NulNul	;
	st	Y+,Temp		;
	swap	Temp		;
	st	Y+,NulNul	;
	st	Y+,NulNul	;
	dec	Cnt		;
	brne	Scale1		; bis 60 Byte gefllt sind
	st	Y+,NulNul	; restliche 4 Byte zu 64 auffllen
	st	Y+,NulNul	;
	st	Y+,NulNul	;
	st	Y+,NulNul	;
	adiw	YL,63		; 128 Byte weiter - nur in jeder 3. Zeile
	adiw	YL,63		;
	adiw	YL,2		; max. 63 im adiw Befehl mglich
	cpi	YH,$20		;
	brne	Scale		;

;*************************************************************************
;* Skala beschriften
;*************************************************************************
	ldi	Cnt,4		;
	clr	YL		;
	ldi	YH,$10		;
	ldi	Char,2		;
Einer:
	clt			;
	rcall	ChrOut		;
	inc	Char		;
	inc	YL		;
	inc	YL		;
	inc	YL		;
	set			;
	rcall	ChrOut		;
	inc	Char		;
	inc	YL		;
	inc	YL		;
	dec	Cnt		;
	brne	Einer		;
	ldi	Cnt,5		;
	clr	Char
Zehner:
	set			;
	push	Char		;
	ldi	Char,1		;
	rcall	ChrOut		;
	pop	Char
	inc	YL		;
	rcall	ChrOut		;
	inc	YL		;
	inc	Char		;
	clt			;
	push	Char		;
	ldi	Char,1		;
	rcall	ChrOut		;
	inc	YL		;
	pop	Char
	rcall	ChrOut		;
	inc	Char		;
	inc	YL		;
	inc	YL		;
	dec	Cnt		;
	brne	Zehner		;
	ldi	Cnt,3		;
	clr	Char
Zwanzg:
	set			;
	push	Char		;
	ldi	Char,2		;
	rcall	ChrOut		;
	pop	Char
	inc	YL		;
	rcall	ChrOut		;
	inc	YL		;
	inc	Char		;
	clt			;
	push	Char		;
	ldi	Char,2		;
	rcall	ChrOut		;
	inc	YL		;
	pop	Char
	rcall	ChrOut		;
	inc	Char		;
	inc	YL		;
	inc	YL		;
	dec	Cnt		;
	brne	Zwanzg		;


;*************************************************************************
;* Interrupt starten
;*************************************************************************
	
	ldi	Temp,$F4	; LSB zuerst,Clk/4,aktiv=~\_, Ruhe=0
	out	SPCR,Temp	; Serial Periph.Interface Control Reg.

	ldi	FadCnt,FdConst	; Video-Fading starten
	set			; T-Flag=1 -> Start it Vertikalimpuls
	clr	ImpFlg
	inc	ImpFlg		; = $01 -> Vertikalimpuls
	sei			; Interrupt Enable Flag

	out	SPDR,NulNul	; Dummy zum SPI-Start ausgeben


;*************************************************************************
;* Hauptprogramm : Schleife fr Dreieckschwingung, 2* 480 Mewerte
;*************************************************************************
Haupt:
	clr	XLo		; X-Koordinate auf Null
	clr	XHi
CntUp:
	rcall	ADDA		; X zum DAC, Y mit ADC messen, zum LCD ausgeben
	inc	XLo
	breq	IncHi
	cpi	XLo,$DF		; 479 dez = $1DF
	brne	CntUp
	tst	XHi		; = $0DF oder $1DF ?
	breq	CntUp
	
CntDn:
	rcall	ADDA		; X zum DAC, Y mit ADC messen, zum LCD ausgeben
	dec	XLo
	cpi	XLo,$FF
	breq	DecHi		; XLo = $FF dann bertrag auf XHi
	tst	XLo
	brne	CntDn		; XLo = 0 ? sonst weiter
	tst	XHi		; = $000 oder $100 ?
	brne	CntDn		; =$100 -> weiter dekrementieren
	breq	CntUp		; =$000 -> inkrementieren
DecHi:
	dec	XHi
	rjmp	CntDn
IncHi:
	inc	XHi
	rjmp	CntUp

;*************************************************************************
;* Unterprogramm gibt zunchst am D/A-Wandler LTC1257 12 Bit seriell aus
;*************************************************************************
ADDA:
	ldi	Cnt,9		; Zhler fr die ersten 9 Datenbits
	mov	Temp,XHi	;
	ror	Temp		; MSB ins Cy
	mov	Temp,XLo	; weitere 8 Bit holen
Loop9:
	brcc	low		; je ein Bit via Cy zum DAC
	sbi	PortB,SDat	; SDat=H
	brcs	high
low:
	cbi	PortB,SDat	; SDat=L
high:
	sbi	PortD,SClk	; DAC-Clock _/~
	rol	Temp		; max Taktfreq. 1,4MHz!
	dec	Cnt		; ndert Cy nicht
	cbi	PortD,SClk	; DAC-Clock ~\_
	brne	Loop9		; 9 Bit erreicht ?

	cbi	PortB,SDat	; unterste 3 Bit=L
	ldi	Cnt,3		;
Loop3:
	sbi	PortD,SClk	; DAC-Clock _/~
	dec	Cnt		;
	cbi	PortD,SClk	; DAC-Clock ~\_
	brne	Loop3		; 3 Bit erreicht ?
	cbi	PortB,LdDac	; DA-Wandler /Load ~\_
	nop
	sbi	PortB,LdDac	; DA-Wandler /Load _/~
;*************************************************************************
;* jetzt 6 Bit vom  AD-Wandler lesen
;*************************************************************************
	cbi	DDRB,SDat	; (2) SDat = input
	cbi	PortD,CsAdc	; (2) ADC-/Enable ~\_
	rcall	ADClk		; (3) zwei ungltige Bit am Anfang
	rcall	ADClk		; (3)
	clr	ADDat		; (1)
	ldi	Cnt,6		; (1) Zhler, nur 6 Bit werden weiterverwendet
Loop6:
	rcall	ADClk		; (3)
	clc			; (1)
	sbic	PinB,SDat	; (2) nchste Zeile berspringen wenn SDat= 0
	sec			; (1) SDat=1 also Cy=1
	rol	ADDat		; (1) SDat ins unterste Bit von ADDat schieben
	dec	Cnt		; (1) Zhler weiterschieben
	brne	Loop6		; (1+1) 
	ldi	Cnt,6		; (1) Zhler fr nicht bentigte 6 Bits
Rest6:
	rcall	ADClk		; (3)
	dec	Cnt		; (2) Zhler weiterschieben
	brne	Rest6		; (4)
	sbi	PortD,CsAdc	; (2) ADC-/Enable _/~
	sbi	DDRB,SDat	; (2) SDat = output

	inc	ADDat		; (1) Meergebnis inkrementieren,
	com	ADDat		; (1) invertieren und
	andi	ADDat,$3F	; (2) auf 6 Bit begrenzen

;*************************************************************************
;* jetzt Ausgabe zum Bildspeicher
;*************************************************************************
	mov	Temp,XHi	; (1)
	mov	ZL,XLo		; (1) unterste 3 Bit ergeben ein Pixel-Byte
	lsr	Temp		; (1) X-Wert hchstwertiges Bit 9(9)
	ror	ZL		; (1) via Cy in Adressregister Z
	com	ZL		; (1) X-Richtung invertieren, fr inv. Op.
	lsr	ADDat		; (1) Y-Wert niederwertigstes Bit 1(6)
	ror	ZL		; (1) via Cy in Adressregister Z
	lsr	ADDat		; (1) Y-Wert Bit 2(6)
	ror	ZL		; (1) via Cy in Adressregister Z
	ldi	ZH,$20		; (2) erster Bildspeicher in 2000...2FFF
	add	ZH,ADDat	; (1) Y-Wert Bits 3-6(6) addieren
	sbiw	ZL,4		; (1) 512-480 = 32 Bit =4 Byte entfallen
	mov	Temp,XLo	; (1) jetzt unterste 3 Bit in 1-aus-8 wandeln
	andi	Temp,$07	; (2) maskieren mit % 0000 0111
	clr	PixByt		; (1) PixByt=00
	sec			; (1) Cy=1
PixLoop:
	ror	PixByt		; (1) ein High-Bit durchschieben
	dec	Temp		; (1) $80/$40/$20/$10/$08/$04/$02/$01
	brge	PixLoop		; (1+1) max 8 mal
;	com	PixByt		; (1) invertieren fr Schwarz auf Wei

	ldi	Cnt,6		; (1) in 6 Bildspeicher ausgeben
VidRam:
	ld	Temp,Z		; (2)
	or	Temp,PixByt	; (1)
	st	Z,Temp		; (2)
	subi	ZH,-$10		; (2) 4 kByte weiter
	dec	Cnt		; (1)
	brne	VidRam		; (1+1)
	ret			; (4)

;*************************************************************************
;* Unterprogramm: ein A/D-Takt fr LTC1286 mit Wartezeit (f-max.=250kHz)
;*************************************************************************
ADClk:
	ldi	Temp,3		; (2) mind. 2 usec warten
Wart1:
	dec	Temp		; (1)
	brne	Wart1		; (1+1)
	sbi	PortD,SClk	; (2) ADC-Clock _/~
	ldi	Temp,5		; (2) mind. 2 usec warten
Wart2:
	dec	Temp		; (1)
	brne	Wart2		; (1+1)
	cbi	PortD,SClk	; (2) ADC-Clock ~\_
	ret			; (4)


;*************************************************************************
;* Unterprogramm schreibt ein Pixel ins Skalen-Ram, restliche Pixel
;* bleiben unberhrt, bergabevariable X_Pix, Y_Pix je ein Byte (MSB von Y=0)
;*************************************************************************
PutPix:
	mov	Temp,X_Pix	; (1) Aufteilen: 8 Pixel * 64 Byte pro Zeile
	lsr	X_Pix		; (1) 
	lsr	X_Pix		; (1)
	lsr	X_Pix		; (1) 5 Bit Byteadresse in X_Pix
	andi	Temp,$07	; (1) untere 3 Bit ergeben Pixelposition
	clr	PixByt		; (1) auf Null
	sec			; (1) Pixel zunchst ins Cy
PixRol:
	rol	PixByt		; (1) Pixel weiterschieben
	dec	Temp		; (1) 
	brge	PixRol		; (1+1) 
	com	PixByt		; (1) schwarzes Pixel auf weiem Hintergrund
	clr	YL		; SRam-Adresse berechnen
	lsr	Y_Pix		; 7Bit-Y-Koordinate aufteilen in 4+3 Bit
	ror	YL		; nchste 3 Bit von links in YL schieben
	lsr	Y_Pix		;
	ror	YL		;
	lsr	Y_Pix		;
	ror	YL		;
	add	YL,X_Pix	; 5 Bit X-Wert addieren
	ldi	YH,$10		; Skalen-Ram $1000..$1FFF
	add	YH,Y_Pix	; obere 4 Bit der Y-Koordinate addieren

	ld	Temp,Y		; 8 Pixel holen
	and	Temp,PixByt	; andere Bits unverndert lassen
	st	Y,Temp
	ret
;*************************************************************************
;* Unterprogramm gibt ein Zeichen ins SRAM
;* bergabevariable:	
;* Y-Register 	= Zieladresse oben links
;* Char		= Zeichen-Nummer, hier 0...9
;* T-Flag=1	= 4 Bit nach rechts in 2 aufeinanderfolgende Bytes versetzt
;*************************************************************************
ChrOut:
	push	Temp
	push	Cnt
	push	Char
	push	YL		; Y retten
	push	YH		;
	ldi	Cnt,7		; 7 Zeilen
	ldi	ZH,high(Tab5x7)	; Zeichenmatrix-Anfang
	ldi	ZL,low(Tab5x7)	;
	lsl	ZL		; 16Bit-Adressierung, daher
	rol	ZH		; Adresse verdoppeln
	clr	Temp		;
	lsl	Char		; 8 Byte pro Zeichen
	rol	Temp		; daher Char * 8
	lsl	Char		;
	rol	Temp		;
	lsl	Char		;
	rol	Temp		;
	add	ZL,Char		; Tabellenanfang fr Char
	adc	ZH,Temp		;
GetLin:
	lpm			; Tabellenwert holen
	brtc	NoShft		; um 4 Bit rechts verschieben ?
	clr	Temp		;
	lsr	LpmReg		;
	ror	Temp		;
	lsr	LpmReg		;
	ror	Temp		;
	lsr	LpmReg		;
	ror	Temp		;
	lsr	LpmReg		;
	ror	Temp		;
	sbiw	YL,1
	ld	Temp2,Y
	or	Temp,Temp2
	st	Y+,Temp

NoShft:
	st	Y+,LpmReg	; ein Byte ins Ram
	adiw	ZL,1		; Zeichen-Tabelle um 1 = eine Matrix-Zeile weiter
	adiw	YL,LinLen-1	; im Ram um eine Displayzeile weiter
	dec	Cnt		; nchste Zeile
	brne	GetLin		; alle 7 Zeilen vorbei?
	pop	YH		;
	pop	YL		;Y holen
	pop	Char
	pop	Cnt
	pop	Temp
	ret



;*************************************************************************
;* Tabelle zur Ziffernanzeige ( rechts nach links, LSB=links MSB=rechts)
;*************************************************************************
Tab5x7:

	.db $1C,$22,$32,$2A,$26,$22,$1C,0	; 0
	.db $08,$0C,$08,$08,$08,$08,$1C,0	; 1
	.db $1C,$22,$20,$10,$08,$04,$3E,0	; 2
	.db $3E,$10,$08,$10,$20,$22,$1C,0	; 3
	.db $10,$18,$14,$12,$3E,$10,$10,0	; 4
	.db $3E,$02,$1E,$20,$20,$22,$1C,0	; 5
	.db $18,$04,$02,$1E,$22,$22,$1C,0	; 6
	.db $3E,$20,$10,$08,$04,$04,$04,0	; 7
	.db $1C,$22,$22,$1C,$22,$22,$1C,0	; 8
	.db $1C,$22,$22,$3C,$20,$10,$0C,0	; 9

;	.db $38,$44,$64,$54,$4C,$44,$38,0	; 0
;	.db $10,$18,$10,$10,$10,$10,$38,0	; 1
;	.db $38,$44,$40,$20,$10,$08,$7C,0	; 2
;	.db $7C,$20,$10,$20,$40,$44,$38,0	; 3
;	.db $20,$30,$28,$24,$7C,$20,$20,0	; 4
;	.db $7C,$04,$3C,$40,$40,$44,$38,0	; 5
;	.db $30,$08,$04,$3C,$44,$44,$38,0	; 6
;	.db $7C,$40,$20,$10,$08,$08,$08,0	; 7
;	.db $38,$44,$44,$38,$44,$44,$38,0	; 8
;	.db $38,$44,$44,$78,$40,$20,$18,0	; 9
